@@ -34,13 +34,14 @@ from re import compile as re_compile, IGNORECASE
 from os.path import basename, abspath
 from time import tzset, sleep, clock
 from gettext import gettext as _
+from urlparse import urlparse
 from os import environ
 from sys import argv
 
 # "If I have seen a little further it is by standing on the shoulders of Giants."
 #                                    --- Isaac Newton
 
-from files import Photograph, GPXLoader
+from files import Photograph, GPXFile, KMLFile
 from utils import get_file, gconf_get, gconf_set
 from utils import Polygon, Struct, make_clutter_color
 from utils import format_list, format_coords, valid_coords, maps_link
@@ -347,6 +348,7 @@ class PreferencesController(CommonAttributes):
 
 class LabelController(CommonAttributes):
     """Control the behavior and creation of ChamplainLabels."""
+    
     def __init__(self):
         self.select_all = get_obj("select_all_button")
         self.selection  = get_obj("photos_view").get_selection()
@@ -412,6 +414,8 @@ class LabelController(CommonAttributes):
 
 
 class ActorController(CommonAttributes):
+    """Controls the behavior of the custom actors I have placed over the map."""
+    
     def __init__(self):
         self.stage = self.map_view.get_stage()
         self.black = Clutter.Box.new(Clutter.BinLayout())
@@ -469,11 +473,17 @@ class GottenGeography(CommonAttributes):
 # File data handling. These methods interact with files (loading, saving, etc)
 ################################################################################
     
+    def file_dragged_in(self, widget, context, x, y, data, info, time):
+        """Load files that have been dragged in."""
+        self.open_files(data.get_uris())
+    
     def open_files(self, files):
         """Attempt to load all of the specified files."""
         self.progressbar.show()
         invalid_files, total = [], len(files)
-        for i, name in enumerate(map(abspath, files), 1):
+        # abspath is used to correct relative paths entered on the commandline,
+        # urlparse is used to correct URIs given from drag&drop operations.
+        for i, name in enumerate([abspath(urlparse(f).path) for f in files], 1):
             self.redraw_interface(i / total, basename(name))
             try:
                 try:            self.load_img_from_file(name)
@@ -511,7 +521,9 @@ class GottenGeography(CommonAttributes):
         """Parse GPX data, drawing each GPS track segment on the map."""
         start_time = clock()
         
-        gpx = GPXLoader(filename, self.gpx_pulse, self.create_polygon)
+        open_file = KMLFile if filename[-3:].lower() == 'kml' else GPXFile
+        gpx = open_file(filename, self.gpx_pulse, self.create_polygon)
+        
         self.status_message(_("%d points loaded in %.2fs.") %
             (len(gpx.tracks), clock() - start_time))
         
@@ -765,6 +777,11 @@ class GottenGeography(CommonAttributes):
             spinbutton.set_value(offset.pop())
         get_obj("open").connect("update-preview", self.update_preview,
             get_obj("preview_label"), get_obj("preview_image"))
+        
+        drop_target = get_obj("photos_and_map")
+        drop_target.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.MOVE)
+        drop_target.connect("drag-data-received", self.file_dragged_in)
+        drop_target.drag_dest_add_uri_targets()
     
     def gpx_sensitivity(self):
         """Control the sensitivity of GPX-related widgets."""
